-- Hittites for VP Functions
-- Author: Jarcast
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
WARN_NOT_SHARED = false; include( "SaveUtils" ); MY_MOD_NAME = "pineappleJar_HittitesVP";

include("FLuaVector.lua")
include("IconSupport.lua")
include("InstanceManager.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local Teams 					= Teams
local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local moreUniques4VPID = "259c3593-34dd-4fdd-bd7d-8ed67cc0797c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingMUfVP  = Game_IsUsingMod(moreUniques4VPID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function IsInteger(n)
  return n==math.floor(n)
end
--==========================================================================================================================
-- GAME DEFINES
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
local CSDecayXTurn = GameDefines.MINOR_FRIENDSHIP_DROP_PER_TURN /(-100)
local CSDecayXTurnHostile = GameDefines.MINOR_FRIENDSHIP_DROP_PER_TURN_HOSTILE /(-100)
local CSDecayXTurnAggressor = GameDefines.MINOR_FRIENDSHIP_DROP_PER_TURN_AGGRESSOR /(-100)
local traitHittitesID = GameInfoTypes["TRAIT_PDAN_HITTITES"]
local g_IsTraitActive = IsTraitActive(traitHittitesID)
local EDubsargisID = GameInfoTypes["BUILDING_PDAN_EDUBSARGIS"]
local dummyEDubsargisID = GameInfoTypes["BUILDING_PDAN_UB1DUMMY"]
local UUAnytID = GameInfoTypes["UNIT_PDAN_ANYT"]

local VictoryThreshold = 25
local PercentAnytUUProductionCSTribute = 0.25
--========================================================================================================================
-- FUNCTIONS
--========================================================================================================================
-- UA: Bond and Oath
--========================================================================================================================
function Jar_HittitesNewUA(iPlayer)
	local pPlayer = Players[iPlayer]
	if not pPlayer:IsAlive() then return end
	if not pPlayer:HasTrait(traitHittitesID) then return end
	local pEraID = pPlayer:GetCurrentEra()
	if pEraID < GameInfo.Eras["ERA_ANCIENT"].ID then return end -- in case someone makes a compatible Prehistoric Eras modmod
	local iValue = 1+ math.max(0, 1+ pEraID - GameInfo.Eras["ERA_MEDIEVAL"].ID )
	for minorID = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local minorCiv = Players[minorID]
		if minorCiv:IsProtectedByMajor(iPlayer) then
			minorCiv:ChangeMinorCivFriendshipWithMajor(iPlayer, iValue)
			if pPlayer:IsHuman() then
				local pCity = minorCiv:GetCapitalCity()
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(pCity:Plot():GetX(), pCity:Plot():GetY()))), "+" .. iValue .. " [ICON_INFLUENCE] from [ICON_STRENGTH] DoP", 2)
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_HittitesNewUA) end

function Jar_WarSupplyAsUnitProduction(ownerID, cityID, unitTypeID)
	local pPlayer = Players[ownerID]
	local iUnitSupply = pPlayer:GetNumUnitsSupplied()
	if HasTrait(pPlayer, traitHittitesID) then
		local city = pPlayer:GetCityByID(cityID)
		if iUnitSupply > city:GetUnitProduction(unitTypeID) then
			city:SetUnitProduction(unitTypeID, iUnitSupply)
		end
	end
	return true
end
if g_IsTraitActive then GameEvents.CityCanTrain.Add(Jar_WarSupplyAsUnitProduction) end
--==========================================================================================================================
-- UB: E DUBSARGIS
--==========================================================================================================================
function Jar_HittitesNewUBEffect(iPlayer, iAgainstTeam, bPacifier)
	local pTeam1Player = Players[iPlayer]
	local pTeam1 = Teams[pTeam1Player:GetTeam()]
	local pTeam2 = Teams[iAgainstTeam]
	local iTeam2Player = pTeam2:GetLeaderID()
	local pTeam2Player = Players[iTeam2Player]
	
	if pTeam1Player:IsMinorCiv() or pTeam2Player:IsMinorCiv() then return end
	--if not (pTeam1Player:HasTrait(traitHittitesID) or pTeam2Player:HasTrait(traitHittitesID)) then return end
	if pTeam1Player:CountNumBuildings(EDubsargisID) <1 and pTeam2Player:CountNumBuildings(EDubsargisID) <1 then return end
	print("Hittites peace deal detected")
	
	--if pTeam1Player:HasTrait(traitHittitesID) then
	if pTeam1Player:CountNumBuildings(EDubsargisID) >0 then
		print("Player1 is the Hittites")
		local warscore = pTeam1Player:GetWarScore(iTeam2Player)
		if warscore > VictoryThreshold then
			LostWarsVSHittites = load( pTeam2Player, "LostWarsVSHittites" );
			if LostWarsVSHittites == nil then LostWarsVSHittites=0 end
			local updatevalue = LostWarsVSHittites +1
			save( pTeam2Player, "LostWarsVSHittites", updatevalue)
			print("War Score Threshold check: ".. warscore .. ". pTeam1Player: " .. pTeam1Player:GetName() .. ", pTeam2Player: " .. pTeam2Player:GetName() .. ". Wars Hittites won: " .. updatevalue .. ".")
			local pCapital1 = pTeam1Player:GetCapitalCity()
			local oldvalue = pCapital1:GetNumRealBuilding(dummyEDubsargisID)
			pCapital1:SetNumRealBuilding( dummyEDubsargisID, oldvalue+1 )
			print("UB1 dummy placed. Total " .. oldvalue+1)
			
			local InfluenceSteal = math.floor(warscore/2 )
			for minorID = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS-1, 1 do
				local minorCiv = Players[minorID]
				if pTeam1:IsHasMet(minorCiv:GetTeam()) or pTeam2:IsHasMet(minorCiv:GetTeam()) then
					minorCiv:ChangeMinorCivFriendshipWithMajor(iPlayer, InfluenceSteal)
					save( pTeam2Player, "WarLoser", -1*InfluenceSteal)
					save( pTeam2Player, "WarLoserAgainstID", iPlayer)
					if minorCiv:GetMinorCivFriendshipLevelWithMajor(pTeam1Player) == 2 then -- allied would declare war
						pTeam2:MakePeace(minorCiv:GetTeam())
					end
				end
			end
			if pTeam1Player:IsHuman() then
				local iCityX, iCityY = pCapital1:GetX(), pCapital1:GetY()
			
				pTeam1Player:AddNotification(NotificationTypes.NOTIFICATION_GOLDEN_AGE_BEGUN_ACTIVE_PLAYER, 
					'The lesser princes acknowledge your great victory.[NEWLINE]+[COLOR_POSITIVE_TEXT]'..InfluenceSteal..'[ENDCOLOR] [ICON_INFLUENCE] Influence with all met City-States.', 
					'Bond and Oath (E Dubsargis)', 
					iCityX, iCityY)
			end
		end

	--elseif pTeam2Player:HasTrait(traitHittitesID) then	
	elseif pTeam2Player:CountNumBuildings(EDubsargisID) >0 then
		print("Player2 is the Hittites")
		local warscore = pTeam2Player:GetWarScore(iPlayer)
		if warscore > VictoryThreshold then
			LostWarsVSHittites = load( pTeam1Player, "LostWarsVSHittites" );
			if LostWarsVSHittites == nil then LostWarsVSHittites=0 end
			local updatevalue = LostWarsVSHittites +1
			save( pTeam1Player, "LostWarsVSHittites", updatevalue)
			print("War Score Threshold check: ".. warscore .. ". pTeam1Player: " .. pTeam1Player:GetName() .. ", pTeam2Player: " .. pTeam2Player:GetName() .. ". Wars Hittites won: " .. updatevalue .. ".")
			local pCapital2 = pTeam2Player:GetCapitalCity()
			local oldvalue = pCapital2:GetNumRealBuilding(dummyEDubsargisID)
			pCapital2:SetNumRealBuilding( dummyEDubsargisID, oldvalue+1 )
			print("UB1 dummy placed. Total " .. oldvalue+1)
			
			local InfluenceSteal = math.floor(warscore/2)
			for minorID = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS-1, 1 do
				local minorCiv = Players[minorID]
				if pTeam1:IsHasMet(minorCiv:GetTeam()) or pTeam2:IsHasMet(minorCiv:GetTeam()) then
					minorCiv:ChangeMinorCivFriendshipWithMajor(iTeam2Player, InfluenceSteal)
					save( pTeam1Player, "WarLoser", -1*InfluenceSteal)
					save( pTeam1Player, "WarLoserAgainstID", iTeam2Player)
					if minorCiv:GetMinorCivFriendshipLevelWithMajor(pTeam2Player) == 2 then -- allied would declare war
						pTeam2:MakePeace(minorCiv:GetTeam())
					end
				end
			end
			if pTeam2Player:IsHuman() then
				local iCityX, iCityY = pCapital2:GetX(), pCapital2:GetY()
			
				pTeam2Player:AddNotification(NotificationTypes.NOTIFICATION_GOLDEN_AGE_BEGUN_ACTIVE_PLAYER, 
					'The lesser princes acknowledge your great victory.[NEWLINE]+[COLOR_POSITIVE_TEXT]'..InfluenceSteal..'[ENDCOLOR] [ICON_INFLUENCE] Influence with all met City-States.', 
					'Bond and Oath (E Dubsargis)', 
					iCityX, iCityY)
			end
		end
	end
end
if g_IsTraitActive then GameEvents.MakePeace.Add(Jar_HittitesNewUBEffect) end

function Jar_WarLoserInfluenceRemoveWorkaround(iPlayer)
	local pPlayer = Players[iPlayer]
	if not pPlayer:IsAlive() then return end
	InfluenceLost = load( pPlayer, "WarLoser" );
	if InfluenceLost == nil then return end
	iHittiteWinner = load( pPlayer, "WarLoserAgainstID" );
	if iHittiteWinner == nil then return end
	print("Loser: " .. pPlayer:GetName() .. ". Influence lost: " .. InfluenceLost .. ". ID Winner Hittites: " .. iHittiteWinner)
	local pHittiteWinner = Players[iHittiteWinner]
	for minorID = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local minorCiv = Players[minorID]
		if Teams[pPlayer:GetTeam()]:IsHasMet(minorCiv:GetTeam()) or Teams[pHittiteWinner:GetTeam()]:IsHasMet(minorCiv:GetTeam()) then
			minorCiv:ChangeMinorCivFriendshipWithMajor(iPlayer, InfluenceLost)
		end
	end
	save( pPlayer, "WarLoser", nil)
	save( pPlayer, "WarLoserAgainstID", nil)
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_WarLoserInfluenceRemoveWorkaround) end
--==========================================================================================================================
-- UU2: ANYT
--==========================================================================================================================
function Jar_CSTributeProductionAnyt(iMajor, iMinor, iYieldNum, iUnitType, iPlotX, iPlotY, iYieldType)
	local pPlayer = Players[iMajor]
	local pMinor = Players[iMinor]
	if not pPlayer:CanTrain(UUAnytID, true) then return end
	local value = 0
	for row in GameInfo.Yields() do
		if row.ID == iYieldType then
			value = value + iYieldNum
		end
	end
	if value < 1 then return end
	local minorX, minorY
	for mCity in pMinor:Cities() do
		if mCity:IsOriginalCapital() then
			minorX, minorY = mCity:Plot():GetX() , mCity:Plot():GetY()
		end
	end
    local distance = 9999
    local cNearestCity = nil
    for pCity in pPlayer:Cities() do
        local pCityPlot = pCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), minorX, minorY)
        if between < distance and pCityPlot:IsCoastalLand( GameDefines.LAKE_MAX_AREA_SIZE +1 ) then
            distance = between
            cNearestCity = pCity
        end
    end
	if cNearestCity == nil then return end
	local prod2add = math.ceil(value*PercentAnytUUProductionCSTribute)
	
	local prodCurrent = cNearestCity:GetUnitProduction(UUAnytID)
	cNearestCity:SetUnitProduction(UUAnytID, (prodCurrent + prod2add))
	if pPlayer:IsHuman() then
		local textstring = "+"..prod2add.. "[ICON_PRODUCTION] Impressment"
		Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(cNearestCity:Plot():GetX(), cNearestCity:Plot():GetY()))), textstring, 4)
	end
end
if g_IsTraitActive and isUsingMUfVP then GameEvents.PlayerBullied.Add(Jar_CSTributeProductionAnyt) end

print("Jarcast-pdan's Hittites functions loaded")
--==========================================================================================================================
--==========================================================================================================================